@@ -1,6 +1,7 @@
 package certrotationcontroller
 
 import (
+	"context"
 	"fmt"
 	"time"
 
@@ -49,6 +50,45 @@ func NewCertRotationController(
 	kubeInformersForNamespaces v1helpers.KubeInformersForNamespaces,
 	eventRecorder events.Recorder,
 	day time.Duration,
+) (*CertRotationController, error) {
+	return newCertRotationController(
+		kubeClient,
+		operatorClient,
+		configInformer,
+		kubeInformersForNamespaces,
+		eventRecorder,
+		day,
+		false,
+	)
+}
+
+func NewCertRotationControllerOnlyWhenExpired(
+	kubeClient kubernetes.Interface,
+	operatorClient v1helpers.StaticPodOperatorClient,
+	configInformer configinformers.SharedInformerFactory,
+	kubeInformersForNamespaces v1helpers.KubeInformersForNamespaces,
+	eventRecorder events.Recorder,
+	day time.Duration,
+) (*CertRotationController, error) {
+	return newCertRotationController(
+		kubeClient,
+		operatorClient,
+		configInformer,
+		kubeInformersForNamespaces,
+		eventRecorder,
+		day,
+		true,
+	)
+}
+
+func newCertRotationController(
+	kubeClient kubernetes.Interface,
+	operatorClient v1helpers.StaticPodOperatorClient,
+	configInformer configinformers.SharedInformerFactory,
+	kubeInformersForNamespaces v1helpers.KubeInformersForNamespaces,
+	eventRecorder events.Recorder,
+	day time.Duration,
+	refreshOnlyWhenExpired bool,
 ) (*CertRotationController, error) {
 	ret := &CertRotationController{
 		networkLister:        configInformer.Config().V1().Networks().Lister(),
@@ -82,14 +122,15 @@ func NewCertRotationController(
 	certRotator, err := certrotation.NewCertRotationController(
 		"AggregatorProxyClientCert",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "aggregator-client-signer",
-			Validity:      30 * defaultRotationDay,
-			Refresh:       15 * defaultRotationDay,
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "aggregator-client-signer",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.GlobalMachineSpecifiedConfigNamespace,
@@ -100,10 +141,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "aggregator-client",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "aggregator-client",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ClientRotation{
 				UserInfo: &user.DefaultInfo{Name: "system:openshift-aggregator"},
 			},
@@ -122,14 +164,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"KubeAPIServerToKubeletClientCert",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "kube-apiserver-to-kubelet-signer",
-			Validity:      1 * 365 * defaultRotationDay, // this comes from the installer
-			Refresh:       8 * 365 * defaultRotationDay, // this means we effectively do not rotate
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "kube-apiserver-to-kubelet-signer",
+			Validity:               1 * 365 * defaultRotationDay, // this comes from the installer
+			Refresh:                8 * 365 * defaultRotationDay, // this means we effectively do not rotate
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -140,10 +183,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "kubelet-client",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "kubelet-client",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ClientRotation{
 				UserInfo: &user.DefaultInfo{Name: "system:kube-apiserver", Groups: []string{"kube-master"}},
 			},
@@ -162,14 +206,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"LocalhostServing",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "localhost-serving-signer",
-			Validity:      10 * 365 * defaultRotationDay, // this comes from the installer
-			Refresh:       8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "localhost-serving-signer",
+			Validity:               10 * 365 * defaultRotationDay, // this comes from the installer
+			Refresh:                8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -180,10 +225,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "localhost-serving-cert-certkey",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "localhost-serving-cert-certkey",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ServingRotation{
 				Hostnames: func() []string { return []string{"localhost", "127.0.0.1"} },
 			},
@@ -202,14 +248,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"ServiceNetworkServing",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "service-network-serving-signer",
-			Validity:      10 * 365 * defaultRotationDay, // this comes from the installer
-			Refresh:       8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "service-network-serving-signer",
+			Validity:               10 * 365 * defaultRotationDay, // this comes from the installer
+			Refresh:                8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -220,10 +267,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "service-network-serving-certkey",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "service-network-serving-certkey",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ServingRotation{
 				Hostnames:        ret.serviceNetwork.GetHostnames,
 				HostnamesChanged: ret.serviceNetwork.hostnamesChanged,
@@ -243,14 +291,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"ExternalLoadBalancerServing",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "loadbalancer-serving-signer",
-			Validity:      10 * 365 * defaultRotationDay, // this comes from the installer
-			Refresh:       8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "loadbalancer-serving-signer",
+			Validity:               10 * 365 * defaultRotationDay, // this comes from the installer
+			Refresh:                8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -261,10 +310,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "external-loadbalancer-serving-certkey",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "external-loadbalancer-serving-certkey",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ServingRotation{
 				Hostnames:        ret.externalLoadBalancer.GetHostnames,
 				HostnamesChanged: ret.externalLoadBalancer.hostnamesChanged,
@@ -284,14 +334,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"InternalLoadBalancerServing",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "loadbalancer-serving-signer",
-			Validity:      10 * 365 * defaultRotationDay, // this comes from the installer
-			Refresh:       8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "loadbalancer-serving-signer",
+			Validity:               10 * 365 * defaultRotationDay, // this comes from the installer
+			Refresh:                8 * 365 * defaultRotationDay,  // this means we effectively do not rotate
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -302,10 +353,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "internal-loadbalancer-serving-certkey",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.TargetNamespace,
+			Name:                   "internal-loadbalancer-serving-certkey",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ServingRotation{
 				Hostnames:        ret.internalLoadBalancer.GetHostnames,
 				HostnamesChanged: ret.internalLoadBalancer.hostnamesChanged,
@@ -365,14 +417,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"KubeControllerManagerClient",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "kube-control-plane-signer",
-			Validity:      60 * defaultRotationDay,
-			Refresh:       30 * defaultRotationDay,
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "kube-control-plane-signer",
+			Validity:               60 * defaultRotationDay,
+			Refresh:                30 * defaultRotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -383,10 +436,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace,
-			Name:      "kube-controller-manager-client-cert-key",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.GlobalMachineSpecifiedConfigNamespace,
+			Name:                   "kube-controller-manager-client-cert-key",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ClientRotation{
 				UserInfo: &user.DefaultInfo{Name: "system:kube-controller-manager"},
 			},
@@ -405,14 +459,15 @@ func NewCertRotationController(
 	certRotator, err = certrotation.NewCertRotationController(
 		"KubeSchedulerClient",
 		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "kube-control-plane-signer",
-			Validity:      60 * defaultRotationDay,
-			Refresh:       30 * defaultRotationDay,
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
+			Namespace:              operatorclient.OperatorNamespace,
+			Name:                   "kube-control-plane-signer",
+			Validity:               60 * defaultRotationDay,
+			Refresh:                30 * defaultRotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
+			Informer:               kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
+			Lister:                 kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
+			Client:                 kubeClient.CoreV1(),
+			EventRecorder:          eventRecorder,
 		},
 		certrotation.CABundleRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -423,10 +478,11 @@ func NewCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		certrotation.TargetRotation{
-			Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace,
-			Name:      "kube-scheduler-client-cert-key",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
+			Namespace:              operatorclient.GlobalMachineSpecifiedConfigNamespace,
+			Name:                   "kube-scheduler-client-cert-key",
+			Validity:               30 * rotationDay,
+			Refresh:                15 * rotationDay,
+			RefreshOnlyWhenExpired: refreshOnlyWhenExpired,
 			CertCreator: &certrotation.ClientRotation{
 				UserInfo: &user.DefaultInfo{Name: "system:kube-scheduler"},
 			},
@@ -442,49 +498,6 @@ func NewCertRotationController(
 	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
-		"KubeAPIServerCertSyncer",
-		certrotation.SigningRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "kube-control-plane-signer",
-			Validity:      60 * defaultRotationDay,
-			Refresh:       30 * defaultRotationDay,
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
-		},
-		certrotation.CABundleRotation{
-			Namespace:     operatorclient.OperatorNamespace,
-			Name:          "kube-control-plane-signer-ca",
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().ConfigMaps(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.OperatorNamespace).Core().V1().ConfigMaps().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
-		},
-		certrotation.TargetRotation{
-			Namespace: operatorclient.TargetNamespace,
-			Name:      "kube-apiserver-cert-syncer-client-cert-key",
-			Validity:  30 * rotationDay,
-			Refresh:   15 * rotationDay,
-			CertCreator: &certrotation.ClientRotation{
-				UserInfo: &user.DefaultInfo{
-					Name:   "system:kube-apiserver-cert-syncer",
-					Groups: []string{"system:masters"},
-				},
-			},
-			Informer:      kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().Secrets(),
-			Lister:        kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().Secrets().Lister(),
-			Client:        kubeClient.CoreV1(),
-			EventRecorder: eventRecorder,
-		},
-		operatorClient,
-	)
-	if err != nil {
-		return nil, err
-	}
-	ret.certRotators = append(ret.certRotators, certRotator)
-
 	return ret, nil
 }
 
@@ -526,16 +539,18 @@ func (c *CertRotationController) RunOnce() error {
 	return utilerrors.NewAggregate(errlist)
 }
 
-func (c *CertRotationController) Run(workers int, stopCh <-chan struct{}) {
+func (c *CertRotationController) Run(ctx context.Context, workers int) {
 	klog.Infof("Starting CertRotation")
 	defer klog.Infof("Shutting down CertRotation")
-	c.WaitForReady(stopCh)
+	c.WaitForReady(ctx.Done())
 
-	go wait.Until(c.runServiceHostnames, time.Second, stopCh)
-	go wait.Until(c.runExternalLoadBalancerHostnames, time.Second, stopCh)
-	go wait.Until(c.runInternalLoadBalancerHostnames, time.Second, stopCh)
+	go wait.Until(c.runServiceHostnames, time.Second, ctx.Done())
+	go wait.Until(c.runExternalLoadBalancerHostnames, time.Second, ctx.Done())
+	go wait.Until(c.runInternalLoadBalancerHostnames, time.Second, ctx.Done())
 
 	for _, certRotator := range c.certRotators {
-		go certRotator.Run(workers, stopCh)
+		go certRotator.Run(ctx, workers)
 	}
+
+	<-ctx.Done()
 }
